//==============================================================================
// Winter in New England
// by Felix Hermansson, Tsunami Studios (hoodncloak@hotmail.com)
// 18 December 2005
//
// Random Map Script for Age of Empires III
// Based on the original New England random map by Ensemble Studios
// Inspired by the Texas Winter map by M0nTy_PyTh0n
//==============================================================================


//==============================================================================
// includes
//==============================================================================

// include mercenaries
include "mercenaries.xs";


//==============================================================================
// function main 
// generates map
//==============================================================================

void main(void)
{
   // Map generation status bar
   rmSetStatusText("",0.01);

   // Randomize numbers of islands
   int whichVariation=-1;
   whichVariation = rmRandInt(1,2);

   // Choose which natives (Iroquois or Cherokee) appear on the map
   int subCiv0=-1;
   int subCiv1=-1;
   int whichNative=rmRandInt(1,4);
   if ( whichNative > 1 )
   {
      subCiv0=rmGetCivID("Iroquois");
      rmEchoInfo("subCiv0 is Iroquois "+subCiv0);

      subCiv1=rmGetCivID("Iroquois");
      rmEchoInfo("subCiv1 is Iroquois "+subCiv1);
   }
   else
   {
      subCiv0=rmGetCivID("Cherokee");
      rmEchoInfo("subCiv0 is Cherokee "+subCiv0);

      subCiv1=rmGetCivID("Cherokee");
      rmEchoInfo("subCiv1 is Cherokee "+subCiv1);
   }
   rmSetSubCiv(0, "Cherokee", true);
   rmSetSubCiv(1, "Iroquois", true);

   // Pick map size
   int playerTiles=12500;
   if (cNumberNonGaiaPlayers >4)
   {
      playerTiles = 11500;
   }

   // Pick default terrain and water
   rmSetBaseTerrainMix("greatlakes_snow");
   rmSetSeaType("hudson bay");
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Some map turbulence...
   rmSetMapElevationParameters(cElevTurbulence, 0.4, 6, 0.7, 5.0);
   rmSetMapElevationHeightBlend(0.2);

   // Initialize Terrain
   rmSetSeaLevel(4.0);
   rmEnableLocalWater(false);
   rmTerrainInitialize("water");
   rmSetMapType("yukon");
   rmSetMapType("water");
   rmSetWorldCircleConstraint(true);
   rmSetWindMagnitude(2.0);
   rmSetLightingSet("Great Lakes Winter");
   rmSetMapType("snow");

   // Make it snow
   rmSetGlobalSnow(0.7);

   // Choose mercs
   chooseMercs();

   // Define some classes. These are used later for constraints.
   int classPlayer=rmDefineClass("player");
   rmDefineClass("classCliff");
   rmDefineClass("classPatch");
   rmDefineClass("classWall");
   int classbigContinent=rmDefineClass("big continent");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("secrets");
   rmDefineClass("socketClass");
   rmDefineClass("nuggets");

   // -------------Define constraints
   // These are used to have objects and areas avoid each other
   
   // Map edge constraints
   int playerEdgeConstraint=rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(6), rmZTilesToFraction(6), 1.0-rmXTilesToFraction(6), 1.0-rmZTilesToFraction(6), 0.01);
   int longPlayerConstraint=rmCreateClassDistanceConstraint("continent stays away from players", classPlayer, 12.0);

   // Player constraints
   int playerConstraint=rmCreateClassDistanceConstraint("player vs. player", classPlayer, 10.0);
   int smallMapPlayerConstraint=rmCreateClassDistanceConstraint("stay away from players a lot", classPlayer, 70.0);

   // Directional constraints
   int Northwestward=rmCreatePieConstraint("northwestMapConstraint", 0.6, 0.6, 0, rmZFractionToMeters(0.9), rmDegreesToRadians(285), rmDegreesToRadians(75));  // 225 135, 300, 45
   int Southeastward=rmCreatePieConstraint("southeastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(90), rmDegreesToRadians(270));

   // Bonus area constraint
   int bigContinentConstraint=rmCreateClassDistanceConstraint("avoid bonus island", classbigContinent, 25.0);

   // Resource avoidance
   int forestConstraint=rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 10.0);
   int avoidDeer=rmCreateTypeDistanceConstraint("food avoids food", "deer", 45.0);
   int avoidFastCoin=rmCreateTypeDistanceConstraint("fast coin avoids coin", "gold", 30.0);
   int avoidCoin=rmCreateTypeDistanceConstraint("coin avoids coin", "gold", 35);
   int avoidNugget=rmCreateTypeDistanceConstraint("nugget avoid nugget", "AbstractNugget", 35.0);
   int avoidPlayerNugget=rmCreateTypeDistanceConstraint("player nugget avoid nugget", "AbstractNugget", 20.0);
   int avoidNuggetSmall=rmCreateTypeDistanceConstraint("nugget avoid nugget small", "AbstractNugget", 10.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int avoidCliffs=rmCreateClassDistanceConstraint("cliff vs. cliff", rmClassID("classCliff"), 30.0);
   int patchConstraint=rmCreateClassDistanceConstraint("patch vs. patch", rmClassID("classPatch"), 5.0);
   int wallConstraint=rmCreateClassDistanceConstraint("wall vs. wall", rmClassID("classWall"), 40.0);

   // Sheep avoid sheep
   int avoidSheep=rmCreateTypeDistanceConstraint("sheep avoids sheep", "sheep", 40.0);

   // Specify true so constraint stays outside of circle (i.e. inside corners)
   int cornerConstraint0=rmCreateCornerConstraint("in corner 0", 0, true);
   int cornerConstraint1=rmCreateCornerConstraint("in corner 1", 1, true);
   int cornerConstraint2=rmCreateCornerConstraint("in corner 2", 2, true);
   int cornerConstraint3=rmCreateCornerConstraint("in corner 3", 3, true);

   // Unit avoidance
   int avoidStartingUnits=rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 20.0);
   int avoidStartingUnitsSmall=rmCreateClassDistanceConstraint("objects avoid starting units small", rmClassID("startingUnit"), 5.0);

   // Ships vs. ships
   int shipVsShip=rmCreateTypeDistanceConstraint("ships avoid ship", "ship", 5.0);

   // Decoration avoidance
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // VP avoidance
   int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 10.0);
   int avoidImportantItem=rmCreateClassDistanceConstraint("secrets etc avoid each other", rmClassID("importantItem"), 30.0);
   int avoidSocket=rmCreateClassDistanceConstraint("avoid sockets", rmClassID("socketClass"), 12.0);

   // Constraint to avoid water
   int avoidWater4 = rmCreateTerrainDistanceConstraint("avoid water", "Land", false, 4.0);
   int avoidWater15 = rmCreateTerrainDistanceConstraint("avoid water 15", "Land", false, 15.0);
   int avoidWater20 = rmCreateTerrainDistanceConstraint("avoid water medium", "Land", false, 20.0);
   int avoidWater30 = rmCreateTerrainDistanceConstraint("avoid water mid-long", "Land", false, 30.0);
   int avoidWater40 = rmCreateTerrainDistanceConstraint("avoid water long", "Land", false, 40.0);

   // New extra stuff for water spawn point avoidance
   int flagLand = rmCreateTerrainDistanceConstraint("flag vs land", "land", true, 20.0);
   int flagVsFlag = rmCreateTypeDistanceConstraint("flag avoid same", "HomeCityWaterSpawnFlag", 15);
   int flagEdgeConstraint = rmCreatePieConstraint("flags stay near edge of map", 0.5, 0.5, rmGetMapXSize()-20, rmGetMapXSize()-10, 0, 0, 0);
   int circleConstraint=rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));
   int whaleLand = rmCreateTerrainDistanceConstraint("whale v. land", "land", true, 20.0);

   // Map Generation Status Bar
   rmSetStatusText("",0.20);

   // Build up big continent - called, unoriginally enough, "big continent"
   int bigContinentID=rmCreateArea("big continent");
   rmSetAreaSize(bigContinentID, 0.52, 0.52);
   rmSetAreaWarnFailure(bigContinentID, false);
   rmAddAreaConstraint(bigContinentID, longPlayerConstraint);
   rmAddAreaToClass(bigContinentID, classbigContinent);
   rmSetAreaSmoothDistance(bigContinentID, 25);
   rmSetAreaMix(bigContinentID, "yukon snow");
   rmSetAreaElevationType(bigContinentID, cElevTurbulence);
   rmSetAreaElevationVariation(bigContinentID, 2.0);
   rmSetAreaBaseHeight(bigContinentID, 6.0);
   rmSetAreaElevationMinFrequency(bigContinentID, 0.09);
   rmSetAreaElevationOctaves(bigContinentID, 3);
   rmSetAreaElevationPersistence(bigContinentID, 0.2);      
   rmSetAreaCoherence(bigContinentID, 0.5);
   rmSetAreaLocation(bigContinentID, 0.5, 0.85);
   rmSetAreaEdgeFilling(bigContinentID, 5);
   rmSetAreaObeyWorldCircleConstraint(bigContinentID, false);
   rmBuildArea(bigContinentID);

   // Map Generation Status Bar
   rmSetStatusText("",0.30);

   // Build up small continent spur called "small continent spur 1"
   int smallContinent1ID=rmCreateArea("small continent spur 1");
   rmSetAreaSize(smallContinent1ID, 0.2, 0.2);
   rmSetAreaWarnFailure(smallContinent1ID, false);
   rmAddAreaConstraint(smallContinent1ID, longPlayerConstraint);
   rmAddAreaToClass(smallContinent1ID, classbigContinent);
   rmSetAreaSmoothDistance(smallContinent1ID, 25);
   rmSetAreaMix(smallContinent1ID, "yukon snow");
   rmSetAreaElevationType(smallContinent1ID, cElevTurbulence);
   rmSetAreaElevationVariation(smallContinent1ID, 2.0);
   rmSetAreaBaseHeight(smallContinent1ID, 6.0);
   rmSetAreaElevationMinFrequency(smallContinent1ID, 0.09);
   rmSetAreaElevationOctaves(smallContinent1ID, 3);
   rmSetAreaElevationPersistence(smallContinent1ID, 0.2);      
   rmSetAreaCoherence(smallContinent1ID, 0.5);
   rmSetAreaLocation(smallContinent1ID, 0.8, 0.6);
   rmSetAreaEdgeFilling(smallContinent1ID, 5);
   rmSetAreaObeyWorldCircleConstraint(smallContinent1ID, false);
   rmBuildArea(smallContinent1ID);

   // Map Generation Status Bar
   rmSetStatusText("",0.35);

   // Build up small continent spur called "small continent spur 2"
   int smallContinent2ID=rmCreateArea("small continent spur 2");
   rmSetAreaSize(smallContinent2ID, 0.2, 0.2);
   rmSetAreaWarnFailure(smallContinent2ID, false);
   rmAddAreaConstraint(smallContinent2ID, longPlayerConstraint);
   rmAddAreaToClass(smallContinent2ID, classbigContinent);
   rmSetAreaSmoothDistance(smallContinent2ID, 25);
   rmSetAreaMix(smallContinent2ID, "yukon snow");
   rmSetAreaElevationType(smallContinent2ID, cElevTurbulence);
   rmSetAreaElevationVariation(smallContinent2ID, 2.0);
   rmSetAreaBaseHeight(smallContinent2ID, 6.0);
   rmSetAreaElevationMinFrequency(smallContinent2ID, 0.09);
   rmSetAreaElevationOctaves(smallContinent2ID, 3);
   rmSetAreaElevationPersistence(smallContinent2ID, 0.2);      
   rmSetAreaCoherence(smallContinent2ID, 0.5);
   rmSetAreaLocation(smallContinent2ID, 0.2, 0.6);
   rmSetAreaEdgeFilling(smallContinent2ID, 5);
   rmSetAreaObeyWorldCircleConstraint(smallContinent2ID, false);
   rmBuildArea(smallContinent2ID);

   // Decide on player placement
   if ( cNumberTeams == 2 )
   {
      rmSetPlacementTeam(0);
      rmPlacePlayersLine(0.2, 0.5, 0.2, 0.8, 0.0, 0.2);
      rmSetPlacementTeam(1);
      rmPlacePlayersLine(0.8, 0.5, 0.8, 0.8, 0.0, 0.2);
   }
   else
   {
      rmSetPlacementSection(0.75, 0.25);
      rmSetTeamSpacingModifier(0.75);
      rmPlacePlayersCircular(0.4, 0.4, 0);
   }

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(100);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i);
      // Assign to the player.
      rmSetPlayerArea(i, id);
      // Set the size.
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      // rmAddAreaConstraint(id, playerConstraint); 
      // rmAddAreaConstraint(id, playerEdgeConstraint); 
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build the areas
   rmBuildAllAreas();

   // Placement order
   // Trade route -> Lakes -> Natives -> Nuggets -> Resources

   // Map Generation Status Bar
   rmSetStatusText("",0.40);

   // TRADE ROUTE
   int tradeRouteID = rmCreateTradeRoute();

   int socketID=rmCreateObjectDef("sockets to dock Trade Posts");
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmAddObjectDefToClass(socketID, rmClassID("socketClass"));
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 8.0);
   if ( cNumberNonGaiaPlayers < 4 )
   {
      rmAddObjectDefConstraint(socketID, avoidWater15); // To make it avoid the water and the cliffs.
   }

   else 
   {
      rmAddObjectDefConstraint(socketID, avoidWater20); // To make it avoid the water and the cliffs even more on larger map
   }

   // Hacky trade route stuff for weird FFA cases, to handle player placement.
   if ( cNumberTeams == 3 || cNumberTeams > 4 )
   {
      rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.8);
   }
   else
   {
      rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.9);
   }
   rmAddTradeRouteWaypoint(tradeRouteID, 0.5, 0.9);

   if ( cNumberNonGaiaPlayers < 4 )
   {
      rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.5, 0.45, 10, 2);
   }
   else
   {
      rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.5, 0.4, 10, 4);
   }
   
   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "dirt");
   if(placedTradeRoute == false)
   {
      rmEchoError("Failed to place trade route"); 
   }

   // Add the meeting poles along the trade route
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   vector socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.0);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.3);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.7);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 1.0);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);


   // FROZEN LAKES
   int lakeClass=rmDefineClass("lake");
   int lakeConstraint=rmCreateClassDistanceConstraint("lake vs. lake", rmClassID("lake"), 20.0);

   int smalllakeID=rmCreateArea("small lake 1");

   // Create constraint (to be used later)
   int avoidLake1 = rmCreateAreaDistanceConstraint("avoid ice lake 1", smalllakeID, 2.0);
   
   if ( cNumberNonGaiaPlayers < 4 )
   {
      rmSetAreaSize(smalllakeID, rmAreaTilesToFraction(600), rmAreaTilesToFraction(600));
   }
   else
   {
      rmSetAreaSize(smalllakeID, rmAreaTilesToFraction(900), rmAreaTilesToFraction(900));
   }
   rmSetAreaMix(smalllakeID, "great_lakes_ice");
   rmSetAreaBaseHeight(smalllakeID, 2.0);
   rmSetAreaMinBlobs(smalllakeID, 2);
   rmSetAreaMaxBlobs(smalllakeID, 3);
   rmSetAreaMinBlobDistance(smalllakeID, 8.0);
   rmSetAreaMaxBlobDistance(smalllakeID, 12.0);
   rmAddAreaToClass(smalllakeID, lakeClass);
   rmAddAreaConstraint(smalllakeID, playerConstraint);
   rmAddAreaConstraint(smalllakeID, avoidTradeRoute);
   rmAddAreaConstraint(smalllakeID, avoidSocket);
   rmSetAreaLocation(smalllakeID, 0.38, 0.65);
   rmAddAreaInfluenceSegment(smalllakeID, 0.4, 0.52, 0.35, 0.68);
   rmSetAreaCoherence(smalllakeID, 0.7);
   rmSetAreaWarnFailure(smalllakeID, false);
   rmBuildArea(smalllakeID);

   int smalllakeID2=rmCreateArea("small lake 2");

   // Create constraint (to be used later)
   int avoidLake2 = rmCreateAreaDistanceConstraint("avoid ice lake 2", smalllakeID2, 2.0);

   if ( cNumberNonGaiaPlayers < 4 )
   {
      rmSetAreaSize(smalllakeID2, rmAreaTilesToFraction(600), rmAreaTilesToFraction(600));
   }
   else
   {
      rmSetAreaSize(smalllakeID2, rmAreaTilesToFraction(900), rmAreaTilesToFraction(900));
   }
   rmSetAreaMix(smalllakeID2, "great_lakes_ice");
   rmSetAreaBaseHeight(smalllakeID2, 2.0);
   rmSetAreaMinBlobs(smalllakeID2, 5);
   rmSetAreaMaxBlobs(smalllakeID2, 8);
   rmSetAreaMinBlobDistance(smalllakeID2, 5.0);
   rmSetAreaMaxBlobDistance(smalllakeID2, 10.0);
   rmAddAreaToClass(smalllakeID2, lakeClass);
   rmAddAreaConstraint(smalllakeID2, playerConstraint);
   rmAddAreaConstraint(smalllakeID2, avoidTradeRoute);
   rmAddAreaConstraint(smalllakeID2, avoidSocket);
   rmSetAreaLocation(smalllakeID2, 0.62, 0.65);
   rmAddAreaInfluenceSegment(smalllakeID2, 0.6, 0.52, 0.65, 0.68);
   rmSetAreaCoherence(smalllakeID2, 0.7);
   rmSetAreaWarnFailure(smalllakeID2, false);
   rmBuildArea(smalllakeID2);

   // Isles of Shoals - these are set in specific locations.
   int bonusIslandID1=rmCreateArea("isle of shoals 1");
   rmSetAreaSize(bonusIslandID1, rmAreaTilesToFraction(450), rmAreaTilesToFraction(450));
   rmSetAreaTerrainType(bonusIslandID1, "yukon\ground4_yuk");
   rmSetAreaMix(bonusIslandID1, "yukon snow");
   rmSetAreaBaseHeight(bonusIslandID1, 6.0);
   rmSetAreaSmoothDistance(bonusIslandID1, 5);
   rmSetAreaWarnFailure(bonusIslandID1, false);
   rmSetAreaMinBlobs(bonusIslandID1, 4);
   rmSetAreaMaxBlobs(bonusIslandID1, 5);
   rmSetAreaMinBlobDistance(bonusIslandID1, 8.0);
   rmSetAreaMaxBlobDistance(bonusIslandID1, 12.0);
   rmSetAreaCoherence(bonusIslandID1, 0.50);
   rmAddAreaConstraint(bonusIslandID1, bigContinentConstraint);
   rmAddAreaConstraint(bonusIslandID1, longPlayerConstraint);

   // this may be the only island!
   if ( whichVariation == 1 )
   {
      rmSetAreaLocation(bonusIslandID1, 0.40, 0.15);
   }
   else
   {
      rmSetAreaLocation(bonusIslandID1, 0.50, 0.15);
   }
   rmBuildArea(bonusIslandID1);

   // Only make the second island half the time.
   if ( whichVariation == 1 )
   {
      int bonusIslandID2=rmCreateArea("isle of shoals 2");
      rmSetAreaSize(bonusIslandID2, rmAreaTilesToFraction(450), rmAreaTilesToFraction(450));
      rmSetAreaTerrainType(bonusIslandID2, "yukon\ground4_yuk");
      rmSetAreaMix(bonusIslandID2, "yukon snow");
      rmSetAreaBaseHeight(bonusIslandID2, 6.0);
      rmSetAreaSmoothDistance(bonusIslandID2, 5);
      rmSetAreaWarnFailure(bonusIslandID2, false);
      rmSetAreaMinBlobs(bonusIslandID2, 4);
      rmSetAreaMaxBlobs(bonusIslandID2, 5);
      rmSetAreaMinBlobDistance(bonusIslandID2, 8.0);
      rmSetAreaMaxBlobDistance(bonusIslandID2, 12.0);
      rmSetAreaCoherence(bonusIslandID2, 0.50);
      rmAddAreaConstraint(bonusIslandID2, bigContinentConstraint);
      rmAddAreaConstraint(bonusIslandID2, longPlayerConstraint);
      rmSetAreaLocation(bonusIslandID2, 0.60, 0.15);
      rmBuildArea(bonusIslandID2);
   }

   // Map Generation Status Bar
   rmSetStatusText("",0.50);

   // NATIVE AMERICANS

   float NativeVillageLoc = rmRandFloat(0,1);
     
   // Natives are always on the mainland
   int iroquoisVillageAID = -1;
   int iroquoisVillageType = rmRandInt(1,3);
   if ( whichNative > 1 )
   {
      iroquoisVillageAID = rmCreateGrouping("iroquois village A", "native iroquois village "+iroquoisVillageType);
   }
   else
   {
      iroquoisVillageAID = rmCreateGrouping("cherokee village A", "native cherokee village "+iroquoisVillageType);
   }
   rmSetGroupingMinDistance(iroquoisVillageAID, 0.0);
   rmSetGroupingMaxDistance(iroquoisVillageAID, rmXFractionToMeters(0.05));
   rmAddGroupingConstraint(iroquoisVillageAID, avoidImpassableLand);
   rmAddGroupingToClass(iroquoisVillageAID, rmClassID("importantItem"));
   rmAddGroupingConstraint(iroquoisVillageAID, avoidTradeRoute);

   rmPlaceGroupingAtLoc(iroquoisVillageAID, 0, 0.35, 0.8); 
   
   int iroquoisVillageBID = -1;
   iroquoisVillageType = rmRandInt(1,3);
   if ( whichNative > 1 )
   {
      iroquoisVillageBID = rmCreateGrouping("iroquois village B", "native iroquois village "+iroquoisVillageType);
   }
   else
   {
      iroquoisVillageBID = rmCreateGrouping("cherokee village B", "native cherokee village "+iroquoisVillageType);
   }
   rmSetGroupingMinDistance(iroquoisVillageBID, 0.0);
   rmSetGroupingMaxDistance(iroquoisVillageBID, rmXFractionToMeters(0.05));
   rmAddGroupingConstraint(iroquoisVillageBID, avoidImpassableLand);
   rmAddGroupingToClass(iroquoisVillageBID, rmClassID("importantItem"));
   rmAddGroupingConstraint(iroquoisVillageBID, avoidTradeRoute);
   
   rmPlaceGroupingAtLoc(iroquoisVillageBID, 0, 0.65, 0.8); 

   // Starting Unit placement
   int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 5.0);
   rmSetObjectDefMaxDistance(startingUnits, 10.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));
   rmAddObjectDefConstraint(startingUnits, avoidAll);

   int startingTCID= rmCreateObjectDef("startingTC");
   if ( rmGetNomadStart())
   {
      rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 0.0);
   }
   else
   {
      rmAddObjectDefItem(startingTCID, "TownCenter", 1, 0.0);
   }
   rmAddObjectDefToClass(startingTCID, rmClassID("startingUnit"));

   int silverType = -1;
   int playerGoldID = -1;

   int StartAreaTreeID=rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(StartAreaTreeID, "TreeGreatLakesSnow", 1, 0.0);
   rmSetObjectDefMinDistance(StartAreaTreeID, 10.0);
   rmSetObjectDefMaxDistance(StartAreaTreeID, 15.0);
   rmAddObjectDefConstraint(StartAreaTreeID, avoidStartingUnitsSmall);

   /* Berry bushes eliminated from winter map and replaced with additional deer
   int StartBerryBushID=rmCreateObjectDef("starting BerryBush");
   rmAddObjectDefItem(StartBerryBushID, "BerryBush", 3, 5.0);
   rmSetObjectDefMinDistance(StartBerryBushID, 10.0);
   rmSetObjectDefMaxDistance(StartBerryBushID, 15.0);
   rmAddObjectDefConstraint(StartBerryBushID, avoidStartingUnitsSmall);
   */

   int StartDeerID=rmCreateObjectDef("starting Deer");
   rmAddObjectDefItem(StartDeerID, "Deer", 4, 5.0);
   rmSetObjectDefMinDistance(StartDeerID, 12.0);
   rmSetObjectDefMaxDistance(StartDeerID, 18.0);
   rmAddObjectDefConstraint(StartDeerID, avoidStartingUnitsSmall);

   int playerNuggetID=rmCreateObjectDef("player nugget");
   rmAddObjectDefItem(playerNuggetID, "nugget", 1, 0.0);
   rmAddObjectDefToClass(playerNuggetID, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(playerNuggetID, 30.0);
   rmSetObjectDefMaxDistance(playerNuggetID, 35.0);
   rmAddObjectDefConstraint(playerNuggetID, avoidStartingUnitsSmall);
   rmAddObjectDefConstraint(playerNuggetID, avoidPlayerNugget);
   rmAddObjectDefConstraint(playerNuggetID, circleConstraint);

   int waterFlagID=-1;
   
   for(i=1; <cNumberPlayers)
   {
      rmClearClosestPointConstraints();
      // Place starting units and a TC!
      rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Everyone gets one ore grouping close by
      silverType = rmRandInt(1,10);
      playerGoldID = rmCreateObjectDef("player silver closer "+i);
      rmAddObjectDefItem(playerGoldID, "mine", 1, 0.0);
      rmAddObjectDefConstraint(playerGoldID, avoidTradeRoute);
      rmAddObjectDefConstraint(playerGoldID, avoidStartingUnitsSmall);
      rmSetObjectDefMinDistance(playerGoldID, 15.0);
      rmSetObjectDefMaxDistance(playerGoldID, 20.0);
      rmPlaceObjectDefAtLoc(playerGoldID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Placing starting deer (replacing berry bushes)
      // rmPlaceObjectDefAtLoc(StartBerryBushID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(StartDeerID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Placing starting trees
      rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(StartAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Nuggets
      rmSetNuggetDifficulty(1, 1);
      rmPlaceObjectDefAtLoc(playerNuggetID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(playerNuggetID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Water flag
      waterFlagID=rmCreateObjectDef("HC water flag "+i);
      rmAddObjectDefItem(waterFlagID, "HomeCityWaterSpawnFlag", 1, 0.0);
      rmAddClosestPointConstraint(flagEdgeConstraint);
      rmAddClosestPointConstraint(flagVsFlag);
      rmAddClosestPointConstraint(flagLand);
      vector TCLocation = rmGetUnitPosition(rmGetUnitPlacedOfPlayer(startingTCID, i));
      vector closestPoint = rmFindClosestPointVector(TCLocation, rmXFractionToMeters(1.0));

      rmPlaceObjectDefAtLoc(waterFlagID, i, rmXMetersToFraction(xsVectorGetX(closestPoint)), rmZMetersToFraction(xsVectorGetZ(closestPoint)));
      rmClearClosestPointConstraints();
   }

   // Map generation status bar
   rmSetStatusText("",0.60);

   // Define and place Nuggets
   rmSetAreaMix(smallContinent2ID, "newengland_grass");
   int nuggetID= rmCreateObjectDef("nugget"); 
   rmAddObjectDefItem(nuggetID, "Nugget", 1, 0.0);
   rmSetObjectDefMinDistance(nuggetID, 0.0);
   rmSetObjectDefMaxDistance(nuggetID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(nuggetID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(nuggetID, avoidNugget);
   rmAddObjectDefConstraint(nuggetID, avoidStartingUnits);
   rmAddObjectDefConstraint(nuggetID, avoidTradeRoute);
   rmAddObjectDefConstraint(nuggetID, avoidSocket);
   rmAddObjectDefConstraint(nuggetID, avoidAll);
   rmAddObjectDefConstraint(nuggetID, avoidWater20);
   rmAddObjectDefConstraint(nuggetID, circleConstraint);
   rmAddObjectDefConstraint(nuggetID, avoidLake1);
   rmAddObjectDefConstraint(nuggetID, avoidLake2);
   rmSetNuggetDifficulty(2, 3);
   rmSetMapType("newEngland");
   rmSetAreaTerrainType(bigContinentID, "new_england\ground1_ne");
   rmSetAreaMix(bigContinentID, "newengland_grass");
   rmPlaceObjectDefInArea(nuggetID, 0, bigContinentID, cNumberNonGaiaPlayers*5);

   // Placement of crates on the bonus islands.
   // DAL - nuggets instead
   int whichCrate=-1;
   whichCrate = rmRandInt(1,3);

   int islandCrateID= rmCreateObjectDef("island crates"); 
   rmSetNuggetDifficulty(4, 4);
   rmAddObjectDefConstraint(islandCrateID, avoidWater4);
   rmAddObjectDefConstraint(islandCrateID, avoidNuggetSmall);
   if ( whichCrate == 1 )
   {
      rmAddObjectDefItem(islandCrateID, "Nugget", 1, 0.0);
   }
   else if ( whichCrate == 2 )
   {
      rmAddObjectDefItem(islandCrateID, "Nugget", 1, 0.0);
   }
   else
   {
      rmAddObjectDefItem(islandCrateID, "Nugget", 1, 0.0);
   }
   

   rmPlaceObjectDefInArea(islandCrateID, 0, bonusIslandID1, 1);
   if ( whichVariation == 1 )
   {
      rmPlaceObjectDefInArea(islandCrateID, 0, bonusIslandID2, 1);
   }

   // Add some trees to the islands
   int islandTreeID= rmCreateObjectDef("island trees"); 
   rmAddObjectDefConstraint(islandTreeID, avoidWater4);
   rmAddObjectDefItem(islandTreeID, "TreeGreatLakesSnow", 1, 0.0);
   rmPlaceObjectDefInArea(islandTreeID, 0, bonusIslandID1, 7);
   if ( whichVariation == 1 )
   {
      rmPlaceObjectDefInArea(islandTreeID, 0, bonusIslandID2, 7);
   }

   // FISH & WHALES
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 20.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 6.0);
   int whaleVsWhaleID=rmCreateTypeDistanceConstraint("whale v whale", "minkeWhale", 25.0);

   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "FishCod", 2, 5.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.0, cNumberNonGaiaPlayers*5);

   int whaleID=rmCreateObjectDef("whale");
   rmAddObjectDefItem(whaleID, "minkeWhale", 1, 9.0);
   rmSetObjectDefMinDistance(whaleID, 0.0);
   rmSetObjectDefMaxDistance(whaleID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(whaleID, whaleVsWhaleID);
   rmAddObjectDefConstraint(whaleID, whaleLand);
   rmPlaceObjectDefAtLoc(whaleID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

   // Map generation status bar
   rmSetStatusText("",0.70); 
 
   // MINES
   int silverID = -1;
   int silverCount = cNumberNonGaiaPlayers*3;   // 3 per player, plus starting one.
   rmEchoInfo("silver count = "+silverCount);

   for(i=0; < silverCount)
   {
      silverType = rmRandInt(1,10);
      silverID = rmCreateObjectDef("silver "+i);
      rmAddObjectDefItem(silverID, "mine", 1, 0.0);
      rmSetObjectDefMinDistance(silverID, 0.0);
      rmSetObjectDefMaxDistance(silverID, rmXFractionToMeters(0.5));

      rmAddObjectDefConstraint(silverID, avoidFastCoin);
      rmAddObjectDefConstraint(silverID, avoidCoin);
      rmAddObjectDefConstraint(silverID, avoidAll);
      rmAddObjectDefConstraint(silverID, avoidImpassableLand);
      rmAddObjectDefConstraint(silverID, avoidTradeRoute);
      rmAddObjectDefConstraint(silverID, avoidSocket);
      rmAddObjectDefConstraint(silverID, avoidStartingUnits);
      rmAddObjectDefConstraint(silverID, avoidWater30);
      rmAddObjectDefConstraint(silverID, avoidLake1);
      rmAddObjectDefConstraint(silverID, avoidLake2);
      rmPlaceObjectDefAtLoc(silverID, 0, 0.5, 0.5);
   }

   // FORESTS
   int forestTreeID = 0;
   int numTries=6*cNumberNonGaiaPlayers;
   int failCount=0;

   for (i=0; <numTries)
   {   
      int forest=rmCreateArea("forest "+i, rmAreaID("big continent"));
      rmSetAreaWarnFailure(forest, false);
      rmSetAreaSize(forest, rmAreaTilesToFraction(200), rmAreaTilesToFraction(250));
      rmSetAreaForestType(forest, "great lakes forest snow");
      rmSetAreaForestDensity(forest, 1.0);
      rmSetAreaForestClumpiness(forest, 0.9);
      rmSetAreaForestUnderbrush(forest, 0.0);
      rmSetAreaCoherence(forest, 0.4);
      rmSetAreaSmoothDistance(forest, 10);
      rmAddAreaToClass(forest, rmClassID("classForest")); 
      rmAddAreaConstraint(forest, forestConstraint);
      rmAddAreaConstraint(forest, avoidAll);
      rmAddAreaConstraint(forest, avoidImpassableLand); 
      rmAddAreaConstraint(forest, avoidTradeRoute);
      rmAddAreaConstraint(forest, avoidStartingUnits);
      rmAddAreaConstraint(forest, avoidSocket);
      rmAddAreaConstraint(forest, avoidLake1);
      rmAddAreaConstraint(forest, avoidLake2);
      if(rmBuildArea(forest)==false)
      {
         // Stop trying once we fail 5 times
         failCount++;
         if(failCount==5)
            break;
      }
      else
         failCount=0; 
   } 
 
   // Map generation status bar
   rmSetStatusText("",0.80);

   // DEER
   int deerID=rmCreateObjectDef("deer herd");
   int bonusChance=rmRandFloat(0, 1);
   
   if(bonusChance<0.5)
   {
      rmAddObjectDefItem(deerID, "deer", rmRandInt(6,8), 4.0);
   }
   else
   {
      rmAddObjectDefItem(deerID, "deer", rmRandInt(7,9), 6.0);
   }

   rmSetObjectDefMinDistance(deerID, 0.0);
   rmSetObjectDefMaxDistance(deerID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(deerID, avoidDeer);
   rmAddObjectDefConstraint(deerID, avoidAll);
   rmAddObjectDefConstraint(deerID, avoidSocket);
   rmAddObjectDefConstraint(deerID, avoidTradeRoute);
   rmAddObjectDefConstraint(deerID, avoidImpassableLand);
   rmAddObjectDefConstraint(deerID, avoidStartingUnits);
   rmAddObjectDefConstraint(deerID, avoidLake1);
   rmAddObjectDefConstraint(deerID, avoidLake2);
   rmSetObjectDefCreateHerd(deerID, true);
   rmPlaceObjectDefInArea(deerID, 0, bigContinentID, cNumberNonGaiaPlayers*5);

   // Map generation status bar
   rmSetStatusText("",0.90);

   // SHEEP
   int sheepID=rmCreateObjectDef("sheep");
   rmAddObjectDefItem(sheepID, "sheep", 2, 4.0);
   rmSetObjectDefMinDistance(sheepID, 0.0);
   rmSetObjectDefMaxDistance(sheepID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(sheepID, avoidSheep);
   rmAddObjectDefConstraint(sheepID, avoidAll);
   rmAddObjectDefConstraint(sheepID, avoidSocket);
   rmAddObjectDefConstraint(sheepID, avoidTradeRoute);
   rmAddObjectDefConstraint(sheepID, longPlayerConstraint);
   rmAddObjectDefConstraint(sheepID, avoidCliffs);
   rmAddObjectDefConstraint(sheepID, avoidImpassableLand);
   rmAddObjectDefConstraint(sheepID, avoidLake1);
   rmAddObjectDefConstraint(sheepID, avoidLake2);
   rmPlaceObjectDefAtLoc(sheepID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*2);

}